/* Developed for Lenovo Educational Board Pass 3
   © Lenovo Created by Joshua Grosser
   v. 2.0 */

// define the LED pin
#define PIN_OUT 15
// define the unit length in ms
#define UNIT_LENGTH 250
// define the button pin to input morse code
#define BUTTON 4
// define the button pin to read morse code
const int buttonTwo = 5;
// determines whether to output to the LED
bool complete = false; 
// message in letters
String finalMsg = ""; 
// used for translation to and from morse code
static String codex = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890 .,?!:;()@&"; 

// array where the translations to morse code are stored
static String dots_and_dashes[] = {
  ".-",
  "-...",
  "-.-.",
  "-..",
  ".",
  "..-.",
  "--.",
  "....",
  "..",
  ".---",
  "-.-",
  ".-..",
  "--",
  "-.",
  "---",
  ".--.",
  "--.-",
  ".-.",
  "...",
  "-",
  "..-",
  "...-",
  ".--",
  "-..-",
  "-.--",
  "--..",
  ".----",
  "..---",
  "...--",
  "....-",
  ".....",
  "-....",
  "--...",
  "---..",
  "----.",
  "-----",
  "       ",
  ".-.-.-",
  "--..--",
  "..--..",
  "-.-...",
  "---...",
  "-.-.-.",
  "-.--.",
  "-.--.-",
  ".--.-.",
  ".-...",
};

void setup() {
  // set the appropriate LEDs and buttons
  Serial.begin(115200);
  pinMode(BUTTON, INPUT);
  pinMode(buttonTwo, INPUT);
  pinMode(PIN_OUT, OUTPUT);
  digitalWrite(PIN_OUT, LOW);
  pinMode(16, OUTPUT);
  digitalWrite(16, LOW);
}

void loop() { 
  if (complete) // outputs the morse code to LED
  {
    complete = false;
    String morseWord = encode(finalMsg);
    for(int i = 0; i <= morseWord.length(); i++)
    {
      switch(morseWord[i])
      {
        case '.': // dit
          digitalWrite(PIN_OUT, HIGH);
          delay(UNIT_LENGTH);
          digitalWrite(PIN_OUT, LOW);
          delay(UNIT_LENGTH);
  
          break;
  
        case '-': // dah
          digitalWrite(PIN_OUT, HIGH);
          delay(UNIT_LENGTH * 3);
          digitalWrite(PIN_OUT, LOW);
          delay(UNIT_LENGTH);
  
          break;
  
        case ' ': // gap
          delay(UNIT_LENGTH);
      }
    }
    digitalWrite(16, HIGH);
    delay(UNIT_LENGTH);
    digitalWrite(16, LOW);
    delay(UNIT_LENGTH * 3);
    finalMsg = "";
    Serial.println();
  } 
  else // takes in morse code from button
  {
    while (digitalRead(buttonTwo) != HIGH)
    {
      String myMsg = GetLetter();
      Translate(myMsg);
    }
    complete = true;
  }
}

// reads input from button to morse code
String GetLetter()
{
  unsigned long START, STOP, TIME;
  String myMsg = "";
  while(true)
  {
    while(digitalRead(BUTTON) == LOW)
    {
      // do nothing until button is pressed
    }
    digitalWrite(PIN_OUT, HIGH);
    START = millis();
    while(digitalRead(BUTTON) == HIGH)
    {
      // do not register input until button is released
    }
    STOP = millis();
    digitalWrite(PIN_OUT, LOW);
    TIME = STOP - START;
    if (TIME > 25)
    {
      myMsg += Dot_or_Dash(TIME);
    }
    while((millis() - STOP) < 750)
    {
      delay(10);
      if (digitalRead(BUTTON) == HIGH)
      {
        break;
      }
    }
    if ((millis() - STOP) >= 750)
    {
      return myMsg;
    }
  }
}

// determine if it is a dot or dash
char Dot_or_Dash (unsigned long TIME)
{
  char DoD;
  if (TIME < 500)
  {
    DoD = '.';
  }
  else
  {
    DoD = '-';
  }
  return DoD;
}

// translates morse code to letters
void Translate (String myMsg)
{
  for (int i = 0; i < 47; i++)
  {
    if (myMsg == dots_and_dashes[i])
    {
      Serial.print(codex[i]);
      finalMsg += codex[i];
      return ;
    }
  }
  Serial.print("ERROR, I cannot understand you!\n");
  return ;
}

// translates letters to morse code
String encode(String string)
{
  size_t i, j;
  String morseWord = "";
  for( i = 0; string[i]; i++ )
  {
    for( j = 0; j < 47; j++ )
    {
      if( string[i] == codex[j] )
      {
        morseWord += dots_and_dashes[j];
        break;
      }
    }
    morseWord += " "; // Add tailingspace to separate the chars
  }
  return morseWord;
}
